import {Component} from '@angular/core'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {Validators} from '@angular/forms'
import {MaterialService} from '../../service/material.service'
import {MaterialTypeService} from '../../../material-type/service/material-type.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {map} from 'rxjs/operators'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-add',
  templateUrl: './add.component.html',
  styleUrls: ['./add.component.sass']
})
export class AddComponent extends ErrorHandlingComponent {
  formFields: FormField[] = [
    {
      name: 'name',
      label: 'Code',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: (errors) => errors.required, message: 'Un code est requis'}
      ]
    },
    {
      name: 'inventoryQuantity',
      label: 'Quantité en inventaire',
      icon: 'beaker-outline',
      type: 'number',
      required: true,
      validator: Validators.min(0),
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une quantité en inventaire est requise'},
        {conditionFn: errors => errors.min, message: 'La quantité doit être supérieure ou égale à 0'}
      ],
      step: '0.01'
    },
    {
      name: 'materialType',
      label: 'Type de produit',
      icon: 'shape-outline',
      type: 'select',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un type de produit est requis'}
      ],
      options$: this.materialTypeService.all.pipe(map(types => types.map(t => {
        return {value: t.id, label: t.name}
      })))
    },
    {
      name: 'simdutFile',
      label: 'Fiche signalitique',
      icon: 'file-outline',
      type: 'file',
      fileType: 'application/pdf'
    }
  ]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'exists-material-name',
    messageProducer: error => `Un produit avec le nom '${error.name}' existe déjà`
  }]

  constructor(
    private materialService: MaterialService,
    private materialTypeService: MaterialTypeService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouveau produit'
  }

  submit(values) {
    this.subscribeAndNavigate(
      this.materialService.save(values.name, values.inventoryQuantity, values.materialType, values.simdutFile),
      '/catalog/material/list'
    )
  }
}
