import {Component, Directive, EventEmitter, Input, Output, TemplateRef, ViewChild, ViewEncapsulation} from '@angular/core'
import {MatDialog, MatDialogRef} from '@angular/material/dialog'

@Directive({
  selector: 'cre-dialog-body'
})
export class CreDialogBody {
}

@Directive()
abstract class CreDialog<D = CreDialogData> {
  @ViewChild(TemplateRef) dialogTemplate: TemplateRef<any>

  @Output() cancel = new EventEmitter<void>();
  @Output() continue = new EventEmitter<void>();

  private dialogRef: MatDialogRef<TemplateRef<any>> | null

  constructor(
    protected dialog: MatDialog
  ) {
  }

  protected abstract get data(): D

  show() {
    this.open()
  }

  onCancel() {
    this.close()
    this.cancel.emit();
  }

  onContinue() {
    this.close()
    this.continue.emit();
  }

  private open() {
    const config = {
      panelClass: 'cre-dialog-panel',
      data: this.data
    }
    this.dialogRef = this.dialog.open(this.dialogTemplate, config)
  }

  private close() {
    this.dialogRef.close()
  }
}

@Component({
  selector: 'cre-prompt-dialog',
  templateUrl: 'prompt.html',
  styleUrls: ['dialogs.scss'],
  encapsulation: ViewEncapsulation.None
})
export class CrePromptDialog extends CreDialog<CrePromptDialogData> {
  @Input() title: string

  protected get data(): CrePromptDialogData {
    return {
      title: this.title
    }
  }
}

abstract class CreDialogData {
  title: string
}

class CrePromptDialogData extends CreDialogData {
}
